// ---------------------------------------------------------------------------
/// The CppMultiStreamHandler shows usage of the CVB method.
/// The example was made for a four camera setup with CoaxPress interface.
/// Topics: MultiStreamHandler, GenICam Nodmap, CVSisoVin with two boards and four cameras.
// ---------------------------------------------------------------------------

#include <iostream>

#include <cvb/device_factory.hpp>
#include <cvb/utilities/system_info.hpp>
#include <cvb/driver/stream.hpp>
#include <cvb/async/multi_stream_handler.hpp>

struct ConnectionInfo
{
  int Port = 0;
  int Board = 0;
  ConnectionInfo(int port, int board)
    : Port(port)
    , Board(board) 
  {}
};

class MyMultiStreamHandler
  :public Cvb::MultiStreamHandler
{


public: 
  MyMultiStreamHandler(const std::vector<Cvb::StreamPtr> & streamVector)
    :MultiStreamHandler(streamVector)
  {

  }

private: 
  void HandleAsyncStream(const std::vector<Cvb::StreamPtr> &streamVector) override
  {
    auto readData = 0;
    for (auto & stream : streamVector)
    {
      auto result = stream->WaitFor<Cvb::RingBufferImage>(std::chrono::milliseconds(500));


      // Do something.


    }
  }

};

//args: Camera Number, Image Height, Image Width, Framerate (Hz)
int main(int argc, char* argv[])
{
  {
  // Check the number of parameters
    if (argc != 4) {
      std::cerr << "argc is: " << argc << std::endl;
      std::cerr << "Usage: CppMyMultiStreamHandler CamNumber ImageWidth ImageHeigt" << std::endl;
      std::cerr << "CamNumber: 1-4" << std::endl;
      std::cerr << "ImageWidht: 1024" << std::endl;
      std::cerr << "ImageHeight: 1024" << std::endl;
      std::cerr << "Example: CppMetadataReader 2 1024 1024" << std::endl;
      std::cerr << "Don't change the order." << std::endl;

      return 1;
    }

    size_t camNumber = static_cast<size_t> (atoi(argv[1]));
    int imageWidth = atoi(argv[2]);
    int imageHeight = atoi(argv[3]);

  
  try
  {
    std::array<ConnectionInfo, 4> connectionArray =
    {
      ConnectionInfo(1, 0),
      ConnectionInfo(0, 0),
      ConnectionInfo(1, 1),
      ConnectionInfo(0, 1)
    };

    std::vector<Cvb::StreamPtr> streams;

    std::vector<Cvb::NodeMapPtr> camNodemaps;
    streams.reserve(camNumber);

    camNodemaps.reserve(camNumber);

    Cvb::String path(CVB_LIT("%CVB%/drivers/CVSiSoMe.vin"));

    path = Cvb::ExpandPath(path);


    bool nodeMapError = false;
    for (int i = 0; i < camNumber; i++)
    {
      auto device = Cvb::DeviceFactory::Open(path, connectionArray[i].Port, connectionArray[i].Board);
      Cvb::NodeMapPtr nodemapPtr = NULL;

      try
      {
        nodemapPtr = device->NodeMap("Device");
        camNodemaps.emplace_back(nodemapPtr);
        std::cout << "Camera nodemap " << i << " received." << std::endl;
      }
      catch (const std::exception& error)
      {
        std::cout << "Camera " << camNumber - i - 1 << " Nodemap not found." << std::endl;
        nodeMapError = true;
      }

      streams.emplace_back(device->Stream());
    }


    if (nodeMapError == false)
    {

      for (auto nodemaps : camNodemaps)
      {
        try
        {
          std::cout << "Setting up camera." <<  std::endl;
          auto enumPtr = nodemaps->Node<Cvb::EnumerationNode>(Cvb::String("Std::AcquisitionMode"));
          enumPtr->SetValue(Cvb::String("CoaxPress"));

          auto nodePtr = nodemaps->Node<Cvb::IntegerNode>(Cvb::String("Std::Height"));
          nodePtr->SetValue(imageHeight);

          nodePtr = nodemaps->Node<Cvb::IntegerNode>(Cvb::String("Std::Width"));
          nodePtr->SetValue(imageWidth);



        }
        catch (const std::exception& error)
        {
          std::cout << "Issue with camera nodmap." << std::endl;
        }
      }
    }


    //MultiStreamHandler can be used because all cameras are triggered synchronuously.
    auto streamHandler = std::make_shared<MyMultiStreamHandler>(streams);


    //Start acquisition on all cameras.
    Cvb::StreamHandlerGuard guard(streamHandler, Cvb::AutoRun::Yes);


    //Wait for keyboard input to end program.
    getchar();

    camNodemaps.clear();


  }
  catch (const std::exception& error)
  {
    std::cout << error.what() << std::endl;
  }
}
}

